/* xmlterm.css: Stylesheet for XUL wrapper ../content/xmlterm.xul */

@import url(chrome://global/skin/);

@namespace html url("http://www.w3.org/1999/xhtml");

/**
 * remove black border from iframe
 */
html|iframe {
 min-width: 10px;
 min-height: 10px;
 width: 100px;
 height: 100px;
 border: none;
}
